#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureIn;
uniform sampler2D texture2;
uniform int isOutline;
uniform int useGlossMap;
uniform vec4 modifyRGB;

varying vec3 N;
varying vec3 v;
varying vec2 tex;

void main (void)
{
  if (isOutline == 1) {
    gl_FragColor = vec4(0.0, 0.0, 0.0, texture2D(textureIn, tex.xy).a);
  } else {
    vec3 L = normalize(gl_LightSource[0].position.xyz - v);
    vec3 E = normalize(-v); // we are in Eye Coordinates, so EyePos is (0,0,0)
    vec3 R = normalize(-reflect(L,N));
    vec4 color = texture2D(textureIn, tex.xy).rgba;
    if (modifyRGB.a > 0) {
        if (color.r < 0.5) {
            color.r = modifyRGB.r - ((0.5 - color.r) * 2);
        } else if (color.r > 0.5) color.r = modifyRGB.r + ((color.r - 0.5) * 2);
        if (color.g < 0.5) {
            color.g = modifyRGB.g - ((0.5 - color.g) * 2);
        } else if (color.g > 0.5) color.g = modifyRGB.g + ((color.g - 0.5) * 2);
        if (color.b < 0.5) {
            color.b = modifyRGB.b - ((0.5 - color.b) * 2);
        } else if (color.b > 0.5) color.b = modifyRGB.b + ((color.b - 0.5) * 2);
        color = clamp(color, 0.0, 1.0);
    }
    //gl_FrontLightModelProduct.sceneColor
    vec4 Iamb = gl_FrontLightProduct[0].ambient;
    vec4 Idiff = gl_FrontLightProduct[0].diffuse * color * max(dot(N,L), 0.0);
    Idiff = clamp(Idiff, 0.0, 1.0);
    vec4 Ispec = gl_FrontLightProduct[0].specular * pow(max(dot(R,E),0.0),0.3*gl_FrontMaterial.shininess);
    Ispec = clamp(Ispec, 0.0, 1.0);
    vec4 finalColor = color * ((vec4(0.5, 0.4, 0.3, color.a) + Iamb + Idiff) / 3 + 0.66) + Ispec;
    if (useGlossMap == 1) {
        vec4 glossColor = texture2D(texture2, R.xy);
        gl_FragColor.rgb = ((color.rgb / 2) * 1) + (((glossColor.rgb) * color.rgb) / 2);
        gl_FragColor.a = finalColor.a;
    } else {
        gl_FragColor = finalColor;
    }
  }
}
